
#
from ..ioc.unit import Unit
from ... import dz,xf
from ..ioc.datas import Datas
'''
id|namespace|ns:
deal_id|deal_ns: 
env_id|env_ns:
envs.pub|envs.pri|envs.ns|envs: {
    
}
confs.pub|confs.pri|confs.ns|confs: [

]
confs.pub|confs.pri|confs.ns|confs: {

}
builds: [
    
]

or

[
    ...
]
'''
def split2list(s, by=','):
    if s is None:
        return [s]
    if not dz.islist(s):
        s = s.split(by)
    return s
def join_nnull(by, *args):
    args = [k for k in args if k is not None and k!=""]
    return by.join(args)
    
def joins(prefix, suffix=None, by='.', split=","):
    prefix = split2list(prefix,split)
    suffix = split2list(suffix,split)
    rst = []
    for pfx in prefix:
        for sfx in suffix:
            rst.append(join_nnull(by, pfx, sfx))
    return rst


class ConfUnit(Unit):
    #key_ns = ('id', 'namespace', 'ns')
    key_ns = joins("id,namespace,ns")
    #key_deal_ns = "deal_id,deal_ns,deal_namespace".split(",")
    key_deal_ns = joins("deal", "id,ns,namespace", "_")
    #key_env_ns = "env_id,env_ns,env_namespace,profile_id,profile_ns,profile_namespace".split(",")
    key_env_ns = joins("env,profile", "id,ns,namespace", "_")
    #key_confs_pub = "confs.pub,confs".split(",")
    key_confs_pub = joins("confs,conf", "pub,")
    #key_confs_pri = "confs.pri,confs.prv".split(",")
    key_confs_pri = joins("confs,conf", "pri,prv")
    #key_confs_ns = "confs.ns,confs.namespace".split(",")
    key_confs_ns = joins("confs,conf", "ns,namespace")
    #key_envs_pub = "envs.pub,envs,profiles,profiles.pub".split(",")
    key_envs_pub = joins("envs,env,profiles,profile", "pub,Pub,P,")
    #key_envs_pri = "envs.pri,envs.prv,profiles.pri,profiles.prv".split(",")
    key_envs_pri = joins("envs,env,profiles,profile", "pri,prv,local,p,l")
    #key_envs_ns = "envs.ns,envs.namespace,profiles.ns,profiles.namespace".split(",")
    key_envs_ns = joins("envs,env,profiles,profile", "ns,namespace,n")
    #key_builds = "builds,build".split(",")
    key_builds = joins("builds,build")
    def init(self, conf, mg):
        if type(conf)==str:
            conf = xf.loads(conf)
        if dz.islist(conf):
            conf = {'confs': conf}
        ns = dz.get_one(conf, self.key_ns)
        deal_ns = dz.get_one(conf, self.key_deal_ns)
        env_ns = dz.get_one(conf, self.key_env_ns)
        super().init(ns, deal_ns, env_ns)
        self.tag_key = None
        self.bind(mg)
        self.load(conf)
    def load_confs(self, confs, tag=None):
        rst = []
        if dz.isdict(confs):
            for k,v in confs.items():
                self.conf_key.fill(v, k)
                rst.append(v)
            confs = rst
        for item in confs:
            self.add_conf(item, tag)
            # key,find = self.conf_key(item)
            # if find:
            #     self.set_conf(key, item, tag)
    def load_envs(self, envs, tag=None):
        for key, val in envs.items():
            self.set_env(key, val, tag)
    def load(self, conf):
        tags = [Datas.Key.Pub, Datas.Key.Ns, Datas.Key.Pri]
        keys = [self.key_confs_pub, self.key_confs_ns, self.key_confs_pri]
        for key, tag in zip(keys, tags):
            tag_confs = dz.get_one(conf, key, [])
            self.load_confs(tag_confs, tag)
        keys = [self.key_envs_pub, self.key_envs_ns, self.key_envs_pri]
        for key, tag in zip(keys, tags):
            tag_envs = dz.get_one(conf, key, {})
            self.load_envs(tag_envs, tag)
        builds = dz.get_one(conf, self.key_builds, [])
        for item in builds:
            self.add_build(item)

pass




from .base import *
from ... import dz,pyz
class MethodEncape(BaseEncape):
    def init(self, unit, method, obj, args, maps):
        super().init()
        self.unit = unit
        self.params = method, obj, args, maps
    def call(self, params=None, **_):
        method, obj, args, maps = self.params
        method = self.obj(method)
        if isinstance(obj, Encape):
            obj = self.obj(obj)
        elif type(obj)==str:
            obj, find = self.unit.get(obj)
        if obj is None and params is not None:
            obj = params.obj
        if type(method)==str:
            assert obj is not None and hasattr(obj, method)
            method = getattr(obj, method)
        args = self.elist2obj(args)
        maps = self.edict2obj(maps)
        return method(*args, **maps)
class MethodDeal(BaseDeal):
    def init(self):
        super().init()
        conf = BaseConf()
        conf.ikey(1, 'method','call,fc,data,key'.split(","), need=1)
        conf.ikey(2, 'obj', 'source,obj,object,src'.split(","))
        conf.ikey(3, 'args')
        conf.ikey(4, 'maps')
        self.update = conf
    def deal(self, conf, unit):
        assert 'method' in conf
        method = conf['method']
        method = self.get_encape(method, unit)
        obj = dz.get(conf, "obj")
        obj = self.get_encape(obj, unit)
        args = self.get_elist(unit, conf, 'args',[])
        maps = self.get_edict(unit, conf, 'maps', {})
        return MethodEncape(unit, method, obj, args, maps)

pass



from .tools import *
import re
import platform 
class Builder:
    def __init__(self, pt=None, npt = ".*\.pyc"):
        self.pt = pt
        self.npt = npt
        self.args = {}
    def default_args(self):
        args = {}
        sys = platform.system().lower()
        args['TARGETOS'] = sys
        args['TARGETARCH']=platform.machine().lower()
        args['TARGETPLATFORM'] = args['TARGETOS']+"/"+args['TARGETARCH']
        args['TARGETVARIANT'] = ""
        self.args.update(args)
    def init_args(self, s_args):
        s_args = s_args.replace("\\", " ")
        pt = "--build-arg\s+([^\s=]+)=([^\s]+)"
        rst = re.findall(pt, s_args)
        for k,v in rst:
            self.args[k] = v
    def replace_args(self, s, args):
        pt1 = "(\$\{([^\{\}\$\s]+)\})"
        pt2 = "(\$([\w\d_]+))"
        for pt in [pt1, pt2]:
            rst = re.findall(pt, s)
            for rp, k in rst:
                if k not in args:
                    continue
                v = args[k]
                s = s.replace(rp, v)
        return s
    def has_image(self, s):
        '''
        判断镜像是否存在
        '''
        assert 0, 'not impl'
    def has_container(s, tag=None):
        '''
            判断容器是否存在
        '''
        assert 0, 'not impl'
    def has_images(self, ts):
        for s in ts:
            if not self.has_image(s):
                return False
        return True
    def default_test(self, tag):
        '''
            默认测试运行指令
        '''
        assert 0, 'not impl'
    def read_config(self, fp):
        try:
            s = fread(fp).decode("utf-8")
        except Exception as exp:
            print(f"[WARN] read '{fp}' error: {exp}")
            s = ""
        arr = s.strip().split("\n")
        tag_froms = []
        cmds = {}
        offset = 0
        curr = None
        args = {}
        args.update(self.args)
        for s in arr:
            s=s.strip()
            if len(s)<2:
                continue
            if s.lower().find("arg")==0:
                tmp = s[3:].strip()
                _i = tmp.find("=")
                if _i<0:
                    continue
                k = tmp[:_i].strip()
                v = tmp[_i+1:].strip()
                args[k] = v
                pass
            if s.lower().find("from")==0:
                arr = s.strip().split(" ")[1:]
                arr = [k.strip() for k in arr if k.strip()!=""]
                lws = [k.lower() for k in arr]
                if 'as' in lws:
                    arr = arr[:lws.index('as')]
                tag_from = arr[-1].split("@")[0]
                tag_from = "/".join(tag_from.split("/")[-2:])
                tar_from = self.replace_args(tag_from, args)
                tag_froms.append(tag_from)
                continue
            elif s[:3] == "#@@":
                if curr is None:
                    continue
                cmds[curr][-1][0]+="\n"+s[3:]
            elif s[:2]=="#@":
                s = s[2:]
                i=s.find("=")
                if i<=0:
                    continue
                key = s[:i].strip()
                curr=key
                val = s[i+1:]
                if key not in cmds:
                    cmds[key] = []
                cmds[key].append([val, offset])
                offset+=1
        if 'test' not in cmds and 'tag' in cmds:
            cmds['test'] = [[self.default_test(cmds['tag'][0][0]), offset]]
        orders = []
        for k, lst in cmds.items():
            for it in lst:
                orders.append(it+[k])
        orders.sort(key = lambda x:x[1])
        return tag_froms, cmds, orders
    def check_read(self, fp):
        if self.pt is not None:
            if len(re.findall(self.pt, fp))>0:
                return True
        if self.npt is not None:
            if len(re.findall(self.npt, fp))>0:
                return False
        return True
    def scan(self, dp, out = {}):
        files = os.listdir(dp)
        #print(f"[DEBUG] scan {dp}, init out: {out}, files: {files}")
        for fp in files:
            fp = os.path.join(dp, fp)
            if not self.check_read(fp):
                #print(f"[DEBUG] check_read False on '{fp}'")
                continue
            if os.path.isfile(fp):
                tfroms, cmds, orders = self.read_config(fp)
                if 'tag' not in cmds:
                    continue
                for tname, _ in cmds['tag']:
                    out[tname] = [fp, tfroms, cmds, orders]
                #out[tname] = [fp, tfroms, test]
            elif os.path.isdir(fp):
                self.scan(fp, out)
        return out
    def exec(self, fp, dp=None, orders=None):
        it = self
        if dp is not None:
            fp = os.path.join(dp, fp)
        if orders is None:
            tfroms, cmds, orders = self.read_config(fp)
        for s, offset, key in orders:
            if key=='exec':
                ret = os.system(s)
                if ret!=0:
                    print(f"system exec '{s}' error: {ret}")
                    return ret
            elif key=='try':
                ret = os.system(s)
                if ret!=0:
                    print(f"[warn] system try '{s}' error: {ret}")
            elif key=='py.exec':
                exec(s)
            elif key=='py.eval':
                ret = eval(s)
                print(f"[DEBUG] py.eval({s}): {ret}")
                if type(ret)==int and ret!=0:
                    print(f"python eval '{s}' error: {ret}")
                    return ret
        return 0
    def builds(self, tag, dirpath = ".", s_args="", maps=None):
        #print(f"[DEBUG] start builds image '{tag}'")
        if self.has_image(tag):
            print(f"image '{tag}' already builded")
            return None, 0
        maps = maps or self.scan(dirpath)
        assert tag in maps, f"{tag} file not found in {dirpath}"
        fp, tfroms, cmds, orders = maps[tag]
        dp = os.path.dirname(fp)
        print(f"[DEBUG] tfroms: {tfroms}")
        for tfrom in tfroms:
            if not self.has_image(tfrom):
                _, ret = self.builds(tfrom, dirpath, s_args, maps)
                if ret!=0:
                    print(f"[WARN] images '{tfrom}' build failed")
                    #return maps, ret
        ret=self.exec(fp, None, orders)
        if ret!=0:
            return maps, ret
        #tags = ",".join([k[0] for k in cmds['tag']])
        ret = self.build(fp, tag, dp, s_args)
        return maps, ret
    def build_cmd(self, fp, tag, dp, s_args=""):
        assert 0, 'not impl'
    def build(self, fp, tag, dp, s_args=""):
        #print(f"[DEBUG] single build image '{tag}', in fp {fp}, dp: {dp}")
        s = self.build_cmd(fp, tag, dp, s_args)
        lines = "="*10
        lines = "\n"+lines+"\n"
        print(lines)
        print(f"@@@ TRY BUILD '{tag}' BY '{s}':")
        print(lines)
        ret = os.system(s)
        print(lines)
        if ret!=0:
            print(f"@@@ error build '{tag}': ret={ret}")
        else:
            print(f"@@@ done build '{tag}'")
        print(lines)
        return ret
    def test(self, tag, dirpath = ".", s_args = None, maps=None):
        maps, ret = self.builds(tag, dirpath, s_args, maps)
        if ret!=0:
            print(f"[WARN] image '{tag}' build failed")
            return ret
        maps = maps or self.scan(dirpath)
        fp, tfroms, cmds, orders = maps[tag]
        ret = 0
        for stest, _ in cmds['test']:
            print(f"[DEBUG] test with '{stest}'")
            ret = os.system(stest)
        return ret
    def demo(self):
        args = sys.argv[1:]
        order = args.pop(0)
        tag = args.pop(0)
        dp = "."
        if len(args)>0:
            dp =args.pop(0)
        more_args = " ".join(args)
        self.init_args(more_args)
        if order == 'test':
            ret = self.test(tag, dp, s_args = more_args)
        elif order == 'build':
            _, ret = self.builds(tag, dp, s_args = more_args)
        elif order == 'exec':
            ret = self.exec(tag, dp)
        else:
            print(f'unknown command "{order}"')
            ret = 123
        return ret

pass



from .. import base
from .. import item

class PrevSptDeal(base.BaseDeal):
    def labels(self):
        return [self.spt]
    def types(self):
        return [self.type]
    def build(self, obj):
        if self.ret is not None:
            obj.val = self.ret
            obj.is_val = 1
            return obj
        return item.null
    def prepare(self, mgs):
        self.spt = mgs.like(self.spt)
        self.l = len(self.spt)
    def deal(self, buffer, arr, mg):
        c = buffer.read(self.l)
        if c != self.spt:
            return False
        spt_pos = (buffer.read_base, buffer.read_base+self.l)
        rm = buffer.full().strip()
        rm_pos = buffer.pos()
        buffer.clean2read(self.l)
        it = item.Item(self.spt, spt_pos, type = self.type, is_val = 0)
        if len(rm)==0:
            if not self.allow_empty or (len(arr)>0 and arr[-1].is_val):    
                arr.append(it)
                return True
        obj = item.Item(rm, rm_pos, type = 'str', is_val = 0)
        arr.append(obj)
        arr.append(it)
        return True
    """
        分隔符，有分隔符后将缓存的数据当作字符串
    """
    def init(self, spt, allow_empty = False, type = "spt", ret=None):
        self.spt = spt
        self.allow_empty = allow_empty
        self.l = len(spt)
        self.type = type
        self.ret = ret

pass


#
from buildz import evalz,xf

from . import build
class VarEval(evalz.Eval):
    def init(self, key, stype = 'list'):
        self.key = key
        self.stype = stype
    def call(self, obj):
        args, maps = obj
        if self.stype=='list':
            return args[self.key]
        return maps[self.key]
class VarBuild(evalz.Build):
    def init(self, stype="list"):
        super().init()
        self.stype = stype
    def call(self, data):
        return VarEval(data['data'][0], self.stype)
lb = VarBuild('list')
db = VarBuild('dict')
evalBuilder = evalz.EvalBuilder.Make().set(["l","list","args"], lb).set(['d','dict','maps'], db)
class EvalBuild(build.Build):
    def call(self, conf):
        judge = xf.g1(conf, judge=None, eval=None)
        if judge is None:
            return None
        return evalBuilder(judge)

pass



from .. import base
from .. import item
from .. import exp
from . import lr
class ListDeal(lr.LRDeal):
    """
        分隔符，有分隔符后将缓存的数据当作字符串
    """
    def init(self, left, right):
        self.sp().init(left, right, 'list')
    def build(self, arr, l_item):
        rst = []
        for _item in arr:
            if not _item.check(is_val = 1):
                raise exp.FormatExp("format error find in list:", _item.pos, _item.val)
            rst.append(_item.val)
        return item.DealItem(rst, l_item.pos, self.id(), is_val = 1, is_list = 1)

pass


from .. import base
from .. import item
from .. import exp
class LRDeal(base.BaseDeal):
    def labels(self):
        return [self.left]
    def types(self):
        return []
    def deal(self, buffer, rst, mg):
        cl = buffer.read(self.ll)
        if self.left!=cl:
            return False
        _arr = []
        rm = buffer.full().strip()
        rm_pos = buffer.pos()
        buffer.clean2read(self.ll)
        if len(rm)>0:
            _arr.append(item.Item(rm, rm_pos, type="str", is_val=False))
        arr_pos = list(rm_pos)
        while True:
            cr = buffer.read(self.lr)
            if cr == self.right:
                rm = buffer.full().strip()
                rm_pos = buffer.pos()
                buffer.clean2read(self.lr)
                break
            if not mg.deal(buffer, _arr):
                e_pos = buffer.pos()
                raise exp.Exp("Error lr", e_pos)
        buffer.clean()
        arr_pos[1] = rm_pos[1]
        arr_pos = tuple(arr_pos)
        if len(rm)>0:
            _arr.append(item.Item(rm , rm_pos, type = 'str', is_val=False))
        if self.mg_build:
            dts = mg.build_arr(_arr)
        else:
            dts = _arr
        obj = self.build_arr(dts, arr_pos)
        rst.append(obj)
        return True
    def to_vals(self, _arr, mg):
        dts = []
        for k in _arr:
            _k = mg.build(k)
            if item.is_null(_k):
                continue
            dts.append(_k)
        return dts
    def build_arr(self, obj, arr_pos):
        return item.Item(obj, arr_pos, type = self.type, is_val=True)
    """
        分隔符，有分隔符后将缓存的数据当作字符串
    """
    def prepare(self, mg):
        self.left = mg.like(self.left)
        self.right = mg.like(self.right)
        self.ll = len(self.left)
        self.lr = len(self.right)
    def init(self, left, right, name= "lr", mg_build = True):
        self.left = left
        self.right = right
        self.ll = len(left)
        self.lr = len(right)
        self.name = name
        self.mg_build = mg_build
    def err(self, s):
        return s.replace("<lr>", self.name)

pass


#coding=utf-8

from buildz import pyz
getch = None
if pyz.is_windows:
    from . import win
    getch = win.Getch()
else:
    from . import lx
    getch = lx.Getch()

pass
def open():
    global getch
    return getch.open()

pass

def close():
    global getch
    getch.close()

pass



import sqlite3
import sys,os
from .basez import SimpleDv, fetch
from .structz import CMD
from buildz import xf,fz
from buildz.db import tls
class Db(SimpleDv):
    # func to impl:
    def to_list(self, rst):
        rows = self.cursor.description
        keys = [k[0].lower() for k in rows]
        result = []
        result.append(keys)
        if rst is None or len(rst)==0:
            return result
        result += rst
        return result
    def new_con(self):
        return sqlite3.connect(self.fp)
    def new_cursor(self):
        return self.con.cursor()
    def clone(self):
        return Db(self.fp, self.as_map)
    def __init__(self, fp, as_map=False, *argv, **maps):
        #def __init__(self, fp):
        self.con = None
        self.cursor = None
        self.as_map = as_map
        self.init(fp)
    def init(self, fp):
        fz.makefdir(fp)
        self.fp = fp
    def sql_tables(self, table = None):
        """
            if not note, use name instead
            require:
                table_name, table_note
        """
        query_table = ""
        if table is not None:
            query_table =  f" and name = '{table}'"
        return f"select name as table_name, '' as table_note from sqlite_master where type='table' {query_table};"
    def sql_indexes(self, table=None, index=None):
        """
            require:
                table_name, index_name, is_unique, index_type, index_note
        """
        query_table = ""
        if table is not None:
            query_table =  f" and tbl_name = '{table}'"
            if index is not None:
                query_table+= f" and name='{index}'"
        return f"select tbl_name as table_name, name as index_name, -1 as is_unique, '?' as index_type, '' as index_note from sqlite_master where type='index' {query_table};"
    def columns(self, table, as_map=None):
        """
            require:
                table_name, column_name, column_type, column_default, nullable, column_offset, column_note
        """
        rst = self.query(f"PRAGMA table_info({table})", as_map=0)
        dts = rst[1:]
        keys = "table_name, column_name, column_type, column_default, nullable, column_offset, column_note".split(", ")
        rst = [keys]
        for dt in dts:
            cid, name, _type, notnull, dflt_value, pk = dt
            tmp = [table, name, _type, dflt_value, notnull, cid, '']
            rst.append(tmp)
        rst = self.out_list_sqlite3(rst, as_map)
        return rst
    def index_keys(self, table, index, as_map=None):
        """
            require:
                table_name, index_name, column_name, index_offset, column_note
        """
        rst = self.query(f"PRAGMA index_info({index})", as_map=0)
        dts = rst[1:]
        keys = "table_name, index_name, column_name, index_offset, column_note".split(", ")
        rst = [keys]
        for dt in dts:
            seqno, cid, name = dt
            tmp = [table, index, name, seqno, '']
            rst.append(tmp)
        return self.out_list_sqlite3(rst, as_map)
    def out_list_sqlite3(self, query_result, as_map=None):
        if as_map is None:
            as_map = self.as_map
        rst = query_result
        if as_map and len(rst)>0:
            if len(rst)==1:
                return []
            keys = rst[0]
            rst = rst[1:]
            rst = [{k:v for k,v in zip(keys, dt)} for dt in rst]
        return rst
    def iou_sql(self, table, ks, vs, sets, qs):
        if len(qs)==0:
            sql = f"insert into {table}({ks}) values({vs})"
        else:
            sql = f"insert into {table}({ks}) values({vs}) ON CONFLICT({','.join(qs)}) DO UPDATE SET {sets}"
        return sql
    def insert_or_updatexxx(self, maps, table, keys = None, update_keys = None):
        if type(maps)!=dict:
            maps = maps.__dict__
        if keys is None:
            keys = []
        if type(keys) not in (list, tuple):
            keys = [keys]
        kvs = [[k,tls.py2sql(v)] for k,v in maps.items()]
        sets = [f"{k}={v}" for k,v in kvs if update_keys is None or k in update_keys]
        sets = ",".join(sets)
        ks = ",".join([kv[0] for kv in kvs])
        vs = ",".join([str(kv[1]) for kv in kvs])
        sql = f"insert into {table}({ks}) values({vs}) ON CONFLICT DO UPDATE SET {sets}"
        #print(f"[TESTZ] sql: {sql}")
        return self.execute(sql)

pass
def build(argv, conf):
    root = xf.g(conf, root=None)
    fp = argv[0]
    if root is not None:
        fp = os.path.join(root, fp)
    as_map = xf.g(conf, as_map=False)
    dv = Db(fp, as_map)
    return dv
def buildbk(argv, conf):
    return CMD(make(argv, conf))

pass



#
from .conf import Conf
from .base import Base
from ... import dz
class Up(Conf):
    def init(self):
        super().init()
        self.key('up', 'parent,parents'.split(","))
    def call(self, conf, unit):
        conf, upd = super().call(conf, unit)
        if 'up' in conf:
            up = conf['up']
            up_conf, tag, find = unit.get_conf(up)
            assert find, f"up '{up}' not found in {conf}"
            assert dz.isdict(up_conf), f"up '{up}' should be dict, but list found: {up_conf}"
            dz.fill(up_conf, conf, 0)
            upd = 1
        return conf, upd

pass

class BaseConf(Up):
    def init(self):
        super().init()
        conf = Conf()
        conf.index(0, 'type', need=1)
        conf.index(1, 'id', need=0)
        conf.index(2, 'single', need=0)
        self.index(0, deal=conf, need=1, dict_out=1)


#
try:
    import requests as rq
except:
    print("need package requests: pip install requests")
    raise
session = rq.session()
from buildz import pathz

path = pathz.Path()
path.set("res", "./res")
fp_cert = path.res("ca.crt")
import sys
ip = "127.0.0.1"
port=9999
args = sys.argv[1:]
if len(args)>0:
    ip = args.pop(0)
if len(args)>0:
    port =int(args.pop(0))
proxies = {
    'http':f'http://{ip}:{port}',
    'https':f'http://{ip}:{port}'
}

url = "https://www.baidu.com"
verify=fp_cert
rp = session.get(url, proxies = proxies,verify=verify)
print(rp,len(rp.content))

url = "http://www.baidu.com"
rp = session.get(url, proxies = proxies,verify=verify)
print(rp,len(rp.content))

url = "https://www.bilibili.com/"
headers = {
    "User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/128.0.0.0 Safari/537.36",
    "Accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
    "Accept-Encoding":"gzip, deflate, br, zstd",
    "Accept-Language":"zh-CN,zh;q=0.9"
}
rp = session.get(url, proxies = proxies,verify=verify,headers=headers)
print(rp,len(rp.content))


#
from .tdata import TagData
from .datas import Datas
from .dataset import Dataset
from ... import dz,pyz
class Varset(Dataset):
    @staticmethod
    def fcs(fc, need_val = True):
        def _fc(kvs, ns=None, tag=None):
            if need_val:
                [fc(key, val, ns, tag) for key,val in kvs.items()]
            else:
                [fc(key, ns, tag) for key in kvs]
        return _fc
    def tag(self, _tag):
        if _tag is None:
            _tag = TagData.Key.Pub
        _tag = TagData.Key.stand(_tag)
        return _tag
    def init(self, ids):
        super().init(ids)
        self.vpops = self.fcs(self.vpop, 0)
        self.vremoves = self.fcs(self.vremove, 0)
        self.vsets = self.fcs(self.vset)
    def vpushs(self, kvs, ns=None,tag=None):
        [self.vpush(key, val, ns, tag) for key, val in kvs.items()]
        return pyz.with_out(lambda :self.vpops(kvs))
    def vset(self, key, val, ns=None, tag=None):
        tag = self.tag(tag)
        self.set(key, [val], ns, tag)
    def vget(self, key, ns=None, tag=None):
        tag = self.tag(tag)
        obj, find_tag, find = self.tget(key, ns, tag)
        if not find or len(obj)==0:
            return None, 0
        return obj[-1], 1
    def vhas(self, key, ns=None, tag=None):
        return self.vget(key, ns, tag)[1]
    def vpush(self, key, val, ns = None, tag = None):
        tag = self.tag(tag)
        obj, find_tag, find = self.tget(key, ns, tag)
        if not find or find_tag != tag:
            obj = []
        obj.append(val)
        self.set(key, obj, ns, tag)
    def vpop(self, key, ns=None, tag=None):
        tag = self.tag(tag)
        obj, find_tag, find = self.tget(key, ns, tag)
        if not find or find_tag != tag:
            return 0
        if len(obj)>0:
            obj.pop(-1)
        return 1
    def vremove(self, key, ns=None, tag=None):
        tag = self.tag(tag)
        obj, find_tag, find = self.tget(key, ns, tag)
        if not find or find_tag != tag:
            return 0
        if tag == TagData.Key.Pub:
            keys = self.ids(ns)+self.ids(key)
            dz.dremove(self.pub, keys)
        else:
            map = self.ns[ns]
            del map[key]
        return 1
        




from buildz import confz, pathz, pyz
import os,sys

dp = os.path.dirname(__file__)

path = pathz.Path()
path.set(None, dp)
def test():
    sys_conf = confz.get_sys_conf(path("argx.js"))
    confz.run(dp, "test.js", sys_conf)

pass

def xtest(conf):
    print(f"[DEBUG] xtest:", conf)

pass

pyz.lc(locals(), test)

'''
python -m buildz.confz_test.test

python -m buildz.confz_test.test -v0
'''



import subprocess, os, sys, re

def mkfc(fc):
    def _fc(fp):
        fp = os.path.expanduser(fp)
        return fc(fp)
    return _fc

pass
isfile = mkfc(os.path.isfile)
exists = mkfc(os.path.exists)
isdir = mkfc(os.path.isdir)
from os import system
def cmd(command):
    """
    获取命令执行的输出结果
    """
    try:
        # 使用subprocess.run捕获输出
        result = subprocess.run(
            command,
            shell=True,
            capture_output=True,
            text=True,
            check=True
        )
        return result.stdout
    except subprocess.CalledProcessError as e:
        return e.stderr
        print(f"命令执行失败: {e.stderr}")
        raise e

pass
def check_tcp(ip, port):
    import socket
    skt=socket.socket()
    skt.settimeout(5.0)
    try:
        skt.connect((ip, port))
        return True
    except:
        return False
    finally:
        skt.close()

pass
def assert_exec(val, cmds):
    if not val:
        return os.system(cmds)

def assert_fc(val, fc, *a, **b):
    #print(f"[DEBUG] assert_fc({val}, {fc}, {a}, {b})")
    if val:
        return
    return fc(*a, **b)

def fread(fp, mode='rb'):
    with open(fp, mode) as f:
        return f.read()

pass